#!/usr/bin/env python3
"""
j2y - a super simple Jinja2 templating command line interface
"""

import io
import os
import sys
import hcl
import json
import yaml
import shutil
import argparse
import platform

from datetime import datetime

from pathlib import Path
from typing import Any, Dict, Tuple, List
from jinja2 import Template, Environment, FileSystemLoader

from .util import parse_extra, tty_size, print_stderr
from .filters import registry as filter_registry


def parse_args() -> argparse.Namespace:
    parser = argparse.ArgumentParser(description=__doc__)
    parser.add_argument('template',
                        type=Path,
                        help='path of the Jinja template')
    parser.add_argument('-c', '--context',
                        type=argparse.FileType('r'),
                        default=sys.stdin,
                        help='input file for template context, defaults to stdin')
    parser.add_argument('-o', '--output',
                        type=argparse.FileType('w'),
                        default=sys.stdout,
                        help='output file, defaults to stdout')
    parser.add_argument('-f', '--format',
                        choices=loaders().keys(),
                        default='yaml',
                        help='input file format')
    parser.add_argument('-x', '--extra', action="append",
                        default=[],
                        help='provide extra variables for template context via cli using key=value pair')
    parser.add_argument('-v', '--verbose', action="store_true",
                        help='print template context to stderr')
    return parser.parse_args()


def get_template(path: Path, env: Environment) -> Template:
    return env.get_template(str(path))


def load_context(fp: io.TextIOWrapper, loader: callable) -> Dict[str, Any]:
    return loader(fp) or {}


def loaders() -> Dict[str, callable]:
    return {
        'yaml': lambda fp: yaml.load(fp.read()),
        'json': lambda fp: json.load(fp),
        'hcl': lambda fp: hcl.load(fp),
    }


def get_loader(format: str) -> callable:
    return loaders()[format]


def create_environment(path: Path) -> Environment:
    env = Environment(
        loader=FileSystemLoader(str(path.absolute()))
    )
    env.filters.update(filter_registry)
    return env


def render_template(template: Path,
                    env: Environment,
                    context: Dict[str, Any]) -> str:
    tpl = get_template(template, env)
    return tpl.render(**context)


def write(output: str, fd: io.TextIOWrapper) -> None:
    fd.write(output)
    fd.write('\n')
    fd.flush()


def default_context() -> Dict[str, Any]:
    return {
        "meta": {
            "date": datetime.utcnow(),
            "platform": dict(platform.uname()._asdict())
        }
    }


def print_context(ctx: Dict[str, Any], width: int = 12) -> None:
    print_stderr(yaml.dump(ctx, default_flow_style=False, indent=2, width=width))
    print_stderr("=" * width)
    print_stderr("")


def main():
    args = parse_args()
    extra = parse_extra(args.extra)
    env = create_environment(Path(os.path.curdir))
    ctx = default_context()
    ctx.update(load_context(args.context, get_loader(args.format)))
    ctx.update(extra)
    if args.verbose:
        print_context(ctx, tty_size()[0])
    write(render_template(args.template, env, ctx), args.output)



import base64


registry = {
    "b64encode": lambda a: base64.urlsafe_b64encode(a.encode("utf-8")).decode("utf-8"),
    "b64decode": lambda a: base64.urlsafe_b64decode(a.encode("utf-8")).decode("utf-8"),
}



import sys
import shutil
import functools

from typing import Dict, List, Tuple


print_stderr = functools.partial(print, file=sys.stderr)


def parse_extra(extra: List[str]) -> Dict:
    return dict(x.split('=') for x in extra)


def tty_size() -> Tuple[int, int]:
    return shutil.get_terminal_size((20, 1))



import os
import io
import re

from pathlib import Path
from setuptools import setup, find_packages


def read(path):
    path = Path(os.path.dirname(__file__)) / path
    with io.open(path, 'r', encoding='utf-8') as f:
        return f.read()


long_description = read('README.rst')
requirements = read('requirements.txt').split('\n')


setup(
    name='j2y',
    url='https://github.com/chaudum/j2y',
    author='Christian Haudum',
    author_email='christian@christianhaudum.at',
    description='Jinja2 Template CLI',
    long_description=long_description,
    platforms=['any'],
    license='Apache License 2.0',
    packages=['j2y'],
    entry_points={
        'console_scripts': [
            'j2y = j2y.cli:main',
        ]
    },
    python_requires='>=3.6',
    install_requires=requirements,
    setup_requires=['setuptools_scm'],
    use_scm_version=True,
    classifiers=[
        'Development Status :: 4 - Beta',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: Apache Software License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
    ],
)
